package java.util.logging;

import sun.misc.JavaLangAccess;
import sun.misc.SharedSecrets;
import java.util.MissingResourceException;
import java.util.Locale;
import java.util.ArrayList;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ResourceBundle;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;
import java.io.Serializable;

public class LogRecord implements Serializable
{
    private static final AtomicLong globalSequenceNumber;
    private static final int MIN_SEQUENTIAL_THREAD_ID = 1073741823;
    private static final AtomicInteger nextThreadId;
    private static final ThreadLocal<Integer> threadIds;
    private Level level;
    private long sequenceNumber;
    private String sourceClassName;
    private String sourceMethodName;
    private String message;
    private int threadID;
    private long millis;
    private Throwable thrown;
    private String loggerName;
    private String resourceBundleName;
    private transient boolean needToInferCaller;
    private transient Object[] parameters;
    private transient ResourceBundle resourceBundle;
    private static final long serialVersionUID = 5372048053134512534L;
    
    private int defaultThreadID() {
        long id = Thread.currentThread().getId();
        if (id < 1073741823L) {
            return (int)id;
        }
        Integer value = (Integer)LogRecord.threadIds.get();
        if (value == null) {
            value = Integer.valueOf(LogRecord.nextThreadId.getAndIncrement());
            LogRecord.threadIds.set(value);
        }
        return (int)value;
    }
    
    public LogRecord(Level level, String message) {
        level.getClass();
        this.level = level;
        this.message = message;
        this.sequenceNumber = LogRecord.globalSequenceNumber.getAndIncrement();
        this.threadID = this.defaultThreadID();
        this.millis = System.currentTimeMillis();
        this.needToInferCaller = true;
    }
    
    public String getLoggerName() {
        return this.loggerName;
    }
    
    public void setLoggerName(String loggerName) {
        this.loggerName = loggerName;
    }
    
    public ResourceBundle getResourceBundle() {
        return this.resourceBundle;
    }
    
    public void setResourceBundle(ResourceBundle resourceBundle) {
        this.resourceBundle = resourceBundle;
    }
    
    public String getResourceBundleName() {
        return this.resourceBundleName;
    }
    
    public void setResourceBundleName(String resourceBundleName) {
        this.resourceBundleName = resourceBundleName;
    }
    
    public Level getLevel() {
        return this.level;
    }
    
    public void setLevel(Level level) {
        if (level == null) {
            throw new NullPointerException();
        }
        this.level = level;
        return;
    }
    
    public long getSequenceNumber() {
        return this.sequenceNumber;
    }
    
    public void setSequenceNumber(long sequenceNumber) {
        this.sequenceNumber = sequenceNumber;
    }
    
    public String getSourceClassName() {
        if (this.needToInferCaller) {
            this.inferCaller();
        }
        return this.sourceClassName;
    }
    
    public void setSourceClassName(String sourceClassName) {
        this.sourceClassName = sourceClassName;
        this.needToInferCaller = false;
    }
    
    public String getSourceMethodName() {
        if (this.needToInferCaller) {
            this.inferCaller();
        }
        return this.sourceMethodName;
    }
    
    public void setSourceMethodName(String sourceMethodName) {
        this.sourceMethodName = sourceMethodName;
        this.needToInferCaller = false;
    }
    
    public String getMessage() {
        return this.message;
    }
    
    public void setMessage(String message) {
        this.message = message;
    }
    
    public Object[] getParameters() {
        return this.parameters;
    }
    
    public void setParameters(Object[] parameters) {
        this.parameters = parameters;
    }
    
    public int getThreadID() {
        return this.threadID;
    }
    
    public void setThreadID(int threadID) {
        this.threadID = threadID;
    }
    
    public long getMillis() {
        return this.millis;
    }
    
    public void setMillis(long millis) {
        this.millis = millis;
    }
    
    public Throwable getThrown() {
        return this.thrown;
    }
    
    public void setThrown(Throwable thrown) {
        this.thrown = thrown;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeByte(1);
        objectOutputStream.writeByte(0);
        if (this.parameters == null) {
            objectOutputStream.writeInt(-1);
            return;
        }
        objectOutputStream.writeInt(this.parameters.length);
        for (int i = 0; i < this.parameters.length; ++i) {
            if (this.parameters[i] == null) {
                objectOutputStream.writeObject(null);
            }
            else {
                objectOutputStream.writeObject(this.parameters[i].toString());
            }
        }
        return;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        int byte1 = objectInputStream.readByte();
        int byte2 = objectInputStream.readByte();
        if (byte1 != 1) {
            throw new IOException("LogRecord: bad version: " + byte1 + "." + byte2);
        }
        int int1 = objectInputStream.readInt();
        if (int1 < -1) {
            throw new NegativeArraySizeException();
        }
        if (int1 == -1) {
            this.parameters = null;
        }
        else if (int1 < 255) {
            this.parameters = new Object[int1];
            for (int j = 0; j < this.parameters.length; ++j) {
                this.parameters[j] = objectInputStream.readObject();
            }
        }
        else {
            ArrayList list = new ArrayList(Math.min(int1, 1024));
            for (int i = 0; i < int1; ++i) {
                list.add(objectInputStream.readObject());
            }
            this.parameters = list.toArray(new Object[list.size()]);
        }
        if (this.resourceBundleName != null) {
            try {
                this.resourceBundle = ResourceBundle.getBundle(this.resourceBundleName, Locale.getDefault(), ClassLoader.getSystemClassLoader());
            }
            catch (MissingResourceException ex) {
                this.resourceBundle = null;
            }
        }
        this.needToInferCaller = false;
        return;
    }
    
    private void inferCaller() {
        this.needToInferCaller = false;
        JavaLangAccess javaLangAccess = SharedSecrets.getJavaLangAccess();
        Throwable t = new Throwable();
        int stackTraceDepth = javaLangAccess.getStackTraceDepth(t);
        int n = 1;
        for (int i = 0; i < stackTraceDepth; ++i) {
            StackTraceElement stackTraceElement = javaLangAccess.getStackTraceElement(t, i);
            String className = stackTraceElement.getClassName();
            int loggerImplFrame = this.isLoggerImplFrame(className);
            if (n != 0) {
                if (!(loggerImplFrame == 0)) {
                    n = 0;
                }
            }
            else if (loggerImplFrame == 0 && !className.startsWith("java.lang.reflect.") && !className.startsWith("sun.reflect.")) {
                this.setSourceClassName(className);
                this.setSourceMethodName(stackTraceElement.getMethodName());
                return;
            }
        }
    }
    
    private boolean isLoggerImplFrame(String s) {
        return s.equals("java.util.logging.Logger") || s.startsWith("java.util.logging.LoggingProxyImpl") || s.startsWith("sun.util.logging.");
    }
    
    static {
        globalSequenceNumber = new AtomicLong(0L);
        nextThreadId = new AtomicInteger(1073741823);
        threadIds = new ThreadLocal();
    }
}
